import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  axisLeft,
  axisBottom,
  line
} from 'd3';

import { lineChart } from "./lineChart";
import { map } from "./map";

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

csv('data.json')
  .then(data => {
  	let title = "Satellites Launches"
  	let lastSelectedCountry = null;
    const countryClickCallback = country => {
      title = "Satellites Launches for " + country;
      if (country == lastSelectedCountry) { country = null; }
      lastSelectedCountry = country;
      var filtered = [];
      if (country) {
        data.forEach(d => {
          if (d["Country of Operator/Owner"] == country) {
            filtered.push(d);
          }
        });
      }
      else {
        filtered = data;
        title = "Satellites Launches";
      }
      lastSelectedYear = null;
      render(filtered, filtered);
    }
    
    let lastSelectedYear = null;
  	const yearSelectCallback = year => {
      title = "Satellites Launches for " + year;
      if (year == lastSelectedYear) { year = null; }
      lastSelectedYear = year;
      var filtered = [];
      if (year) {
        data.forEach(d => {
          if (new Date(d["Date of Launch"]).getFullYear() == year) {
            filtered.push(d);
          };
        });
      }
      else {
        filtered = data;
        title = "Satellites Launches";
      }
      lastSelectedCountry = null;
      render(data, filtered);
    };

    let lineGraph = lineChart(null, {svg, height: 200, width, yLocation: 300, yearSelectCallback});
    const renderMap = map(null, {svg, height:350, width, yLocation: 0, countryClickCallback});
  
  	svg.append('text')
      .attr('class', 'title')
      .attr('transform', `translate(200,490)`)
      .text(title);

    const render = (lineData, mapData) => {
      svg.select('.title').text(title);
      lineGraph.render(lineData);
      renderMap(mapData, lastSelectedCountry);
    };
  	render(data, data);
});