import
{
	select,
	json,
	tsv,
	geoPath,
	geoNaturalEarth1,
	scaleLinear,
	max,
	values,
	interpolateBlues
}
from 'd3';
import
{
	feature
}
from 'topojson';

import { colorLegend } from './colorLegend';

export const map = (selection, props) =>
{
	const
	{
		svg,
		height,
		width,
		yLocation,
    countryClickCallback
	} = props;

	const projection = geoNaturalEarth1().scale(height / Math.PI);
	const pathGenerator = geoPath().projection(projection);

	const g = svg.append('g');

	g.attr('transform', `translate(0,-50)`)
	.append('path')
	.attr('class', 'sphere')
	.attr('d', pathGenerator({type: 'Sphere'}));

	let countries = null;
	let countryNames = null;
	let lastData = [];
  let lastClickedCountry = null;

	var render = function ()
	{
		if (!countries || !lastData)
			return;
		let upper = max(values(lastData));
		let colorScale = scaleLinear()
      .domain([0, 50])
      .range(["rgb(255,255,255)", "rgb(255,0,0)"]);
    
    const colorLegendG = svg.append('g')
        .attr('transform', `translate(100,150)`);

    colorLegendG.call(colorLegend, {
      colorScale,
      circleRadius: 8,
      spacing: 20,
      textOffset: 12,
      backgroundRectWidth: 60,
      onClick: d => {
        selectedCountry = (selectedCountry == d ? null : d);
        render(data);
      }
    });
    
		g.selectAll('.country').remove();

		g.selectAll('path').data(countries.features)
		.enter().append('path')
		.attr('class', 'country')
		.attr('d', pathGenerator)
		.attr('fill', d =>
		{
			let v = lastData[countryNames[d.id]] | 0;
			return colorScale(v);
		})
		.on('click', (d, x, y) => {
      let selectedCountry = countryNames[d.id];
      countryClickCallback(selectedCountry);
    })
		.append('title')
		.text(d => {
      let country = countryNames[d.id];
      if (!lastClickedCountry || country == lastClickedCountry) {
        country += " - " + (lastData[country] | 0);
      }
      return country;
    });
	};

	Promise.all([
			tsv('https://unpkg.com/world-atlas@1.1.4/world/50m.tsv'),
			json('https://unpkg.com/world-atlas@1.1.4/world/50m.json')
		]).then(([tsvData, topoJSONdata]) =>
	{

		countryNames = tsvData.reduce((accumulator, d) =>
			{
				accumulator[d.iso_n3] = d.name;
				return accumulator;
			}, {}
			);

		countries = feature(topoJSONdata, topoJSONdata.objects.countries);

		render();
	});

    
  const formatMapData = data => {
    var formatted = [];
    data.forEach(d => {
      var country = d["Country of Operator/Owner"];
      if (!formatted[country]) formatted[country] = 0;
      formatted[country]++;
    });

    return formatted;
  };
  
	return (data, lastSelectedCountry) => {
    	lastClickedCountry = lastSelectedCountry;
      lastData = formatMapData(data);
      render();
    };
}
